package MOS_CV4
  connector BloodFlowConnector
    flow Real Q(unit = "ml/s") "blood flow in ml/sec";
    Real Pressure(unit = "torr") "Pressure in torr";
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(origin = {1, -1}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid, extent = {{-101, 101}, {99, -99}})}));
  end BloodFlowConnector;

  connector BloodFlowInflow
    flow Real Q(unit = "ml/s") "blood flow in ml/sec";
    Real Pressure(unit = "torr") "Pressure in torr";
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid, extent = {{-100, 100}, {100, -100}})}));
  end BloodFlowInflow;

  connector BloodFlowOutflow
    flow Real Q(unit = "ml/s") "blood flow in ml/sec";
    Real Pressure(unit = "torr") "Pressure in torr";
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(fillColor = {238, 238, 238}, fillPattern = FillPattern.Solid, extent = {{-100, 100}, {100, -100}})}));
  end BloodFlowOutflow;

  partial model BloodFlowOnePort
    Real PressureDrop(unit = "torr");
    Real BloodFlow(unit = "ml/s");
    BloodFlowOutflow Outflow annotation(Placement(visible = true, transformation(origin = {90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    BloodFlowInflow Inflow annotation(Placement(visible = true, transformation(origin = {-90, -2}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
  equation
    PressureDrop = Inflow.Pressure - Outflow.Pressure;
    Inflow.Q + Outflow.Q = 0;
    //nehromadí se krev!!
    BloodFlow = Inflow.Q;
    annotation(Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})));
  end BloodFlowOnePort;

  model BloodResisitor
    extends MOS_CV4.BloodFlowOnePort;
    parameter Real BloodResistance(start = 1, unit = "torr.s/ml") "resistance in torr sec/ml";
  equation
    PressureDrop = BloodFlow * BloodResistance;
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(fillColor = {204, 204, 204}, fillPattern = FillPattern.Solid, extent = {{-80, 40}, {80, -40}}), Text(origin = {18, 32}, extent = {{-42, -60}, {8, -2}}, textString = "R"), Text(origin = {8, 58}, extent = {{-60, 24}, {52, -18}}, textString = "%name")}));
  end BloodResisitor;

  model VariableBloodResistor
    extends MOS_CV4.BloodFlowOnePort;
    Modelica.Blocks.Interfaces.RealInput BloodResistance(unit = "torr.s/ml") annotation(Placement(visible = true, transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {0, 60}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
  equation
    PressureDrop = BloodFlow * BloodResistance;
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(fillColor = {179, 179, 179}, fillPattern = FillPattern.Solid, extent = {{-80, 40}, {80, -40}}), Text(origin = {0, 3}, extent = {{-46, 33}, {46, -33}}, textString = "R"), Text(origin = {8, -63}, extent = {{-88, 37}, {58, -25}}, textString = "%name")}));
  end VariableBloodResistor;

  model Inductor
    extends MOS_CV4.BloodFlowOnePort;
    Modelica.Blocks.Interfaces.RealInput Inertance(unit = "torr.s2/ml") annotation(Placement(visible = true, transformation(origin = {0, 88}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
  equation
    PressureDrop = der(BloodFlow) * Inertance;
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(lineThickness = 1, extent = {{-80, 60}, {80, -60}}), Rectangle(origin = {-1, 2}, fillColor = {255, 0, 0}, fillPattern = FillPattern.Solid, extent = {{-39, 52}, {39, -52}}), Text(origin = {-5, 51}, extent = {{-73, 25}, {79, -47}}, textString = "Inertance"), Text(origin = {-2, -75}, extent = {{-112, 19}, {112, -19}}, textString = "%name")}));
  end Inductor;

  model BloodElasticCompartment
    BloodFlowInflow Inflow annotation(Placement(visible = true, transformation(origin = {-90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    BloodFlowOutflow Outflow annotation(Placement(visible = true, transformation(origin = {90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {94, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    parameter Real V0(unit = "ml") = 1;
    Real TransmuralPressure(unit = "torr");
    Modelica.Blocks.Interfaces.RealInput Elastance(unit = "torr/ml") annotation(Placement(visible = true, transformation(origin = {60, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {76, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
    Modelica.Blocks.Interfaces.RealInput ExternalPressure(unit = "torr") annotation(Placement(visible = true, transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {0, 116}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
    Modelica.Blocks.Interfaces.RealInput UnstressedVolume(unit = "ml") annotation(Placement(visible = true, transformation(origin = {-60, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {-80, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
    Modelica.Blocks.Interfaces.RealOutput Pressure(unit = "torr") annotation(Placement(visible = true, transformation(origin = {60, -80}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {80, -80}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
    Modelica.Blocks.Interfaces.RealOutput StressedVolume(start = V0, unit = "ml") annotation(Placement(visible = true, transformation(origin = {0, -80}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {2, -118}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
    Modelica.Blocks.Interfaces.RealOutput Volume(start = V0, unit = "ml") annotation(Placement(visible = true, transformation(origin = {-60, -80}, extent = {{-20, -20}, {20, 20}}, rotation = -90), iconTransformation(origin = {-80, -80}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
  equation
    Inflow.Pressure = Outflow.Pressure;
    Inflow.Pressure = Pressure;
    TransmuralPressure = Pressure - ExternalPressure;
    der(Volume) = Inflow.Q + Outflow.Q;
    StressedVolume = Volume - UnstressedVolume;
    if StressedVolume > 0 then
      TransmuralPressure = Elastance * StressedVolume;
    else
      TransmuralPressure = 0;
    end if;
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Ellipse(origin = {3, -13}, fillColor = {188, 188, 188}, fillPattern = FillPattern.Solid, extent = {{-103, 113}, {97, -87}}, endAngle = 360), Text(origin = {-109, 74}, extent = {{-29, 12}, {21, -14}}, textString = "UV"), Text(origin = {-23, 46}, extent = {{-29, 12}, {63, -38}}, textString = "%name"), Text(origin = {5, 82}, extent = {{-29, 12}, {21, -14}}, textString = "Pext"), Text(origin = {157, 60}, extent = {{-75, 26}, {21, -14}}, textString = "Elastance"), Text(origin = {-65, -44}, extent = {{-51, 16}, {21, -14}}, textString = "Volume"), Text(origin = {47, -98}, extent = {{-111, 34}, {21, -14}}, textString = "StressedVolume"), Text(origin = {97, -44}, extent = {{-51, 14}, {21, -14}}, textString = "Pressure")}));
  end BloodElasticCompartment;

  model Valve
    BloodFlowInflow bloodFlowInflow annotation(Placement(visible = true, transformation(origin = {-90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {-80, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    BloodFlowOutflow bloodFlowOutflow annotation(Placement(visible = true, transformation(origin = {90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0), iconTransformation(origin = {90, 0}, extent = {{-10, -10}, {10, 10}}, rotation = 0)));
    Real q;
    Real dp;
    Boolean open(start = true);
    Real passableVariable;
  equation
    bloodFlowInflow.Q + bloodFlowOutflow.Q = 0;
    q = bloodFlowInflow.Q;
    dp = bloodFlowInflow.Pressure - bloodFlowOutflow.Pressure;
    open = passableVariable > 0;
    if open then
      dp = 0;
      q = passableVariable;
    else
      dp = passableVariable;
      q = 0;
    end if;
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(origin = {67, -1}, fillColor = {255, 85, 127}, fillPattern = FillPattern.Solid, extent = {{-13, 81}, {13, -79}}), Polygon(origin = {-14, -1.01}, fillColor = {255, 85, 127}, fillPattern = FillPattern.Solid, points = {{-64, 81.0114}, {-64, -78.9886}, {66, -18.9886}, {66, 21.0114}, {-64, 81.0114}, {-64, 81.0114}}), Text(origin = {-6, 91}, extent = {{-48, 17}, {48, -17}}, textString = "%name")}));
  end Valve;

  model VariableBloodConductance
    extends MOS_CV4.BloodFlowOnePort;
    Modelica.Blocks.Interfaces.RealInput BloodConductance annotation(Placement(visible = true, transformation(origin = {0, 80}, extent = {{-20, -20}, {20, 20}}, rotation = -90)));
  equation
    PressureDrop * BloodConductance = BloodFlow;
    annotation(Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2}), graphics = {Rectangle(fillColor = {179, 179, 179}, fillPattern = FillPattern.Solid, extent = {{-80, 40}, {80, -40}}), Text(origin = {0, 3}, extent = {{-46, 33}, {46, -33}}, textString = "G"), Text(origin = {8, -63}, extent = {{-88, 37}, {58, -25}}, textString = "%name")}));
  end VariableBloodConductance;
  annotation(Icon(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})), Diagram(coordinateSystem(extent = {{-100, -100}, {100, 100}}, preserveAspectRatio = true, initialScale = 0.1, grid = {2, 2})));
end MOS_CV4;